@@ -102,6 +102,9 @@ BEGIN_EVENT_TABLE(PanoPanel, wxPanel)
     EVT_CHOICE ( XRCID("pano_choice_blender"),PanoPanel::BlenderChanged )
     EVT_BUTTON ( XRCID("pano_button_blender_opts"),PanoPanel::OnBlenderOptions )
 
+    EVT_CHOICE ( XRCID("pano_choice_edgefill"), PanoPanel::OnEdgeFillChanged)
+    EVT_BUTTON ( XRCID("pano_button_fill_opts"), PanoPanel::OnEdgeFillOptions)
+
     EVT_CHOICE ( XRCID("pano_choice_file_format"),PanoPanel::FileFormatChanged )
     EVT_CHOICE ( XRCID("pano_choice_hdr_file_format"),PanoPanel::HDRFileFormatChanged )
 //    EVT_SPINCTRL ( XRCID("pano_output_normal_opts_jpeg_quality"),PanoPanel::OnJPEGQualitySpin )
@@ -219,6 +222,7 @@ bool PanoPanel::Create(wxWindow *parent, wxWindowID id, const wxPoint& pos, cons
     m_BlenderChoice = XRCCTRL(*this, "pano_choice_blender", wxChoice);
     DEBUG_ASSERT(m_BlenderChoice);
     FillBlenderList(m_BlenderChoice);
+    m_edgeFillChoice = XRCCTRL(*this, "pano_choice_edgefill", wxChoice);
 
     m_StitchButton = XRCCTRL(*this, "pano_button_stitch", wxButton);
     DEBUG_ASSERT(m_StitchButton);
@@ -511,6 +515,8 @@ void PanoPanel::UpdateDisplay(const HuginBase::PanoramaOptions & opt, const bool
     XRCCTRL(*this, "pano_button_blender_opts", wxButton)->Show(m_guiLevel>GUI_SIMPLE);
     XRCCTRL(*this, "pano_text_blender", wxStaticText)->Enable(blenderEnabled);
     XRCCTRL(*this, "pano_text_blender", wxStaticText)->Show(m_guiLevel>GUI_SIMPLE);
+    m_edgeFillChoice->SetSelection(static_cast<int>(opt.edgeFillMode));
+    XRCCTRL(*this, "pano_button_fill_opts", wxButton)->Enable(m_edgeFillChoice->GetSelection() != 0);
 
     bool fusionEnabled = (opt.outputLDRExposureBlended || opt.outputLDRExposureLayersFused || opt.outputLDRStacks) && m_guiLevel>GUI_SIMPLE;
     m_FusionChoice->Enable(fusionEnabled);
@@ -919,6 +925,37 @@ void PanoPanel::OnBlenderOptions(wxCommandEvent & e)
     };
 }
 
+void PanoPanel::OnEdgeFillChanged(wxCommandEvent & e)
+{
+    HuginBase::PanoramaOptions opt = pano->getOptions();
+    HuginBase::PanoramaOptions::EdgeFillMode newMode = static_cast<HuginBase::PanoramaOptions::EdgeFillMode>(e.GetSelection());
+    {
+        if (opt.edgeFillMode != newMode)
+        {
+            opt.edgeFillMode = newMode;
+            PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd(*pano, opt)
+            );
+        };
+    };
+}
+
+void PanoPanel::OnEdgeFillOptions(wxCommandEvent& e)
+{
+    HuginBase::PanoramaOptions opt = pano->getOptions();
+    wxDialog dlg;
+    wxXmlResource::Get()->LoadDialog(&dlg, this, wxT("filledge_options_dialog"));
+    wxCheckBox* keepUnfillVersion = XRCCTRL(dlg, "edgefill_save_input", wxCheckBox);
+    keepUnfillVersion->SetValue(opt.keepEdgeFillInput);
+    dlg.CentreOnParent();
+
+    if (dlg.ShowModal() == wxID_OK)
+    {
+        opt.keepEdgeFillInput = keepUnfillVersion->IsChecked();
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::SetPanoOptionsCmd(*pano, opt));
+    };
+}
+
 void PanoPanel::FusionChanged(wxCommandEvent & e)
 {
     int fusion = m_FusionChoice->GetSelection();
@@ -1045,7 +1082,7 @@ void PanoPanel::DoCalcOptimalROI(wxCommandEvent & e)
     };
 };
 
-void PanoPanel::DoStitch()
+void PanoPanel::DoStitch(const wxString& userDefinedSetting)
 {
     if (pano->getNrOfImages() == 0) {
         return;
@@ -1075,13 +1112,7 @@ void PanoPanel::DoStitch()
     }
     DEBUG_DEBUG("tmp PTO file: " << (const char *)currentPTOfn.mb_str(wxConvLocal));
     // copy is not enough, need to adjust image path names...
-    std::ofstream script(currentPTOfn.mb_str(HUGIN_CONV_FILENAME));
-    HuginBase::UIntSet all;
-    if (pano->getNrOfImages() > 0) {
-        fill_set(all, 0, pano->getNrOfImages()-1);
-    }
-    pano->printPanoramaScript(script, pano->getOptimizeVector(), pano->getOptions(), all, false, "");
-    script.close();
+    pano->WritePTOFile(std::string(currentPTOfn.mb_str(HUGIN_CONV_FILENAME)));
 
 //    wxCommandEvent dummy;
 //    MainFrame::Get()->OnSaveProject(dummy);
@@ -1154,6 +1185,10 @@ void PanoPanel::DoStitch()
     wxString switches(wxT(" --delete -o "));
     if(wxConfigBase::Get()->Read(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE) == 1)
         switches=wxT(" --overwrite")+switches;
+    if (!userDefinedSetting.IsEmpty())
+    {
+        switches = " --user-defined-output=" + hugin_utils::wxQuoteFilename(userDefinedSetting) + " " + switches;
+    };
     wxString command = hugin_stitch_project + switches + hugin_utils::wxQuoteFilename(dlg.GetPath()) + wxT(" ") + hugin_utils::wxQuoteFilename(currentPTOfn);
     
     wxConfigBase::Get()->Flush();
@@ -1172,7 +1207,7 @@ void PanoPanel::DoStitch()
     HuginBase::LensDB::SaveLensDataFromPano(*pano);
 }
 
-void PanoPanel::DoSendToBatch()
+void PanoPanel::DoSendToBatch(const wxString& userDefinedSetting)
 {
     if (pano->getNrOfImages() == 0)
     {
@@ -1194,22 +1229,37 @@ void PanoPanel::DoSendToBatch()
     if (wxConfigBase::Get()->Read(wxT("/Processor/start"), HUGIN_PROCESSOR_START) != 0)
     {
         switches += wxT("-b ");
-    }
+    };
     if (wxConfigBase::Get()->Read(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE) != 0)
     {
         switches += wxT("-o ");
-    }
+    };
     if (wxConfigBase::Get()->Read(wxT("/Processor/verbose"), HUGIN_PROCESSOR_VERBOSE) != 0)
     {
         switches += wxT("-v ");
-    }
+    };
+    if (!userDefinedSetting.IsEmpty())
+    {
+        switches += "--user-defined=" + hugin_utils::wxQuoteFilename(userDefinedSetting) + " ";
+    };
     if(pano->isDirty())
     {
         bool showDlg=wxConfigBase::Get()->Read(wxT("ShowSaveMessage"), 1l)==1;
         if(showDlg)
         {
+            // show information that project file needs to be saved first
+            // and user has to give the output prefix afterwards
             wxDialog dlg;
             wxXmlResource::Get()->LoadDialog(&dlg, this, wxT("stitch_message_dlg"));
+            if (!(MainFrame::Get()->getProjectName().IsEmpty()))
+            {
+                // project file was already saved with a name, adapt message text for this use case
+                XRCCTRL(dlg, "stitch_message_text", wxStaticText)->SetLabel(
+                    wxString::Format(_("The project %s will be automatically saved before it can be sent to the batch processor PTBatcherGUI.\nYou will then be asked for the prefix for the panorama output.\n\nYou may need to start the stitching in the batch processor\n PTBatcherGUI if it doesn't start automatically."),
+                        MainFrame::Get()->getProjectName().c_str()
+                    )
+                );
+            };
             if(dlg.ShowModal())
             {
                 if(XRCCTRL(dlg, "stitch_dont_show_checkbox", wxCheckBox)->IsChecked())
@@ -1322,7 +1372,7 @@ void PanoPanel::DoUserDefinedStitch(const wxString& settings)
         wxString path = config->Read(wxT("/userDefinedOutputPath"), MainFrame::Get()->GetDataPath());
         wxFileDialog userOutputDlg(this, _("Select user defined output"),
             path, wxT(""), _("User defined output|*.executor"),
-            wxFD_OPEN | wxFD_MULTIPLE | wxFD_FILE_MUST_EXIST, wxDefaultPosition);
+            wxFD_OPEN | wxFD_FILE_MUST_EXIST, wxDefaultPosition);
         if (userOutputDlg.ShowModal() != wxID_OK)
         {
             return;
@@ -1341,123 +1391,10 @@ void PanoPanel::DoUserDefinedStitch(const wxString& settings)
             return;
         };
     };
-
-    // create a copy, if we need to update the crop setting
-    // save project
-    // copy pto file to temporary file
-    wxString tempDir = wxConfigBase::Get()->Read(wxT("tempDir"), wxT(""));
-    if (!tempDir.IsEmpty())
-    {
-        if (tempDir.Last() != wxFileName::GetPathSeparator())
-        {
-            tempDir.Append(wxFileName::GetPathSeparator());
-        }
-    };
-    wxString currentPTOfn = wxFileName::CreateTempFileName(tempDir + wxT("huginpto_"));
-    if (currentPTOfn.empty())
-    {
-        wxMessageBox(_("Could not create temporary project file"), _("Error"),
-            wxCANCEL | wxICON_ERROR, this);
-        return;
-    }
-    DEBUG_DEBUG("tmp PTO file: " << (const char *)currentPTOfn.mb_str(wxConvLocal));
-    // copy is not enough, need to adjust image path names...
-    std::ofstream script(currentPTOfn.mb_str(HUGIN_CONV_FILENAME));
-    HuginBase::UIntSet all;
-    fill_set(all, 0, pano->getNrOfImages() - 1);
-    pano->printPanoramaScript(script, pano->getOptimizeVector(), pano->getOptions(), all, false, "");
-    script.close();
-
-    //    wxCommandEvent dummy;
-    //    MainFrame::Get()->OnSaveProject(dummy);
-
-#if defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
-    // HuginStitchProject inside main bundle
-    wxString hugin_stitch_project = MacGetPathToBundledAppMainExecutableFile(CFSTR("HuginStitchProject.app"));
-    if (hugin_stitch_project == wxT(""))
-    {
-        DEBUG_ERROR("hugin_stitch_project could not be found in the bundle.");
-        return;
-    }
-    hugin_stitch_project = hugin_utils::wxQuoteFilename(hugin_stitch_project);
-#elif defined __WXMAC__
-    // HuginStitchProject installed in INSTALL_OSX_BUNDLE_DIR
-    wxFileName hugin_stitch_project_app(wxT(INSTALL_OSX_BUNDLE_DIR), wxEmptyString);
-    hugin_stitch_project_app.AppendDir(wxT("HuginStitchProject.app"));
-    CFStringRef stitchProjectAppPath = MacCreateCFStringWithWxString(hugin_stitch_project_app.GetFullPath());
-    wxString hugin_stitch_project = MacGetPathToMainExecutableFileOfBundle(stitchProjectAppPath);
-    CFRelease(stitchProjectAppPath);
-#else
-    wxString hugin_stitch_project = wxT("hugin_stitch_project");
-#endif
-
-    // Derive a default output prefix from the project filename if set, otherwise default project filename
-    wxFileName outputPrefix(getDefaultOutputName(MainFrame::Get()->getProjectName(), *pano));
-    outputPrefix.Normalize();
-
-    // Show a file save dialog so user can confirm/change the prefix.
-    // (We don't have to worry about overwriting existing files, since hugin_switch_project checks this.)
-    // TODO: The following code is similar to stitchApp::OnInit in hugin_switch_project.cpp. Should be refactored.
-    // TODO: We should save the output prefix somewhere, so we can recall it as the default if the user stitches this project again.
-    wxFileDialog dlg(this, _("Specify output prefix"),
-        outputPrefix.GetPath(), outputPrefix.GetName(), wxT(""),
-        wxFD_SAVE, wxDefaultPosition);
-    if (dlg.ShowModal() != wxID_OK)
-    {
-        return;
-    };
-    while (containsInvalidCharacters(dlg.GetPath()))
-    {
-        wxArrayString list;
-        list.Add(dlg.GetPath());
-        ShowFilenameWarning(this, list);
-        if (dlg.ShowModal() != wxID_OK)
-            return;
-    };
-    wxFileName prefix(dlg.GetPath());
-    while (!prefix.IsDirWritable())
-    {
-        wxMessageBox(wxString::Format(_("You have no permissions to write in folder \"%s\".\nPlease select another folder for the final output."), prefix.GetPath().c_str()),
-#ifdef __WXMSW__
-            wxT("Hugin"),
-#else
-            wxT(""),
-#endif
-            wxOK | wxICON_INFORMATION);
-        if (dlg.ShowModal() != wxID_OK)
-        {
-            return;
-        };
-        prefix = dlg.GetPath();
-    };
-    // check free space
-    if (!CheckFreeSpace(prefix.GetPath()))
-    {
-        return;
-    };
-
-    wxString switches(wxT(" --user-defined-output=") + hugin_utils::wxQuoteFilename(userOutputSequence.GetFullPath()) + wxT(" --delete -o "));
-    if (wxConfigBase::Get()->Read(wxT("/Processor/overwrite"), HUGIN_PROCESSOR_OVERWRITE) == 1)
-        switches = wxT(" --overwrite") + switches;
-    wxString command = hugin_stitch_project + switches + hugin_utils::wxQuoteFilename(dlg.GetPath()) + wxT(" ") + hugin_utils::wxQuoteFilename(currentPTOfn);
-
-    wxConfigBase::Get()->Flush();
-#ifdef __WXGTK__
-    // work around a wxExecute bug/problem
-    // (endless polling of fd 0 and 1 in hugin_stitch_project)
-    wxProcess *my_process = new wxProcess(this);
-    my_process->Redirect();
-
-    // Delete itself once processes terminated.
-    my_process->Detach();
-    wxExecute(command, wxEXEC_ASYNC, my_process);
-#else
-    wxExecute(command);
-#endif
+    DoStitchOrSendBatch(userOutputSequence.GetFullPath());
 }
 
-
-void PanoPanel::OnDoStitch ( wxCommandEvent & e )
+void PanoPanel::DoStitchOrSendBatch(const wxString& userDefinedSetting)
 {
     long t;
     if(wxGetKeyState(WXK_COMMAND))
@@ -1466,17 +1403,17 @@ void PanoPanel::OnDoStitch ( wxCommandEvent & e )
     }
     else
     {
-        wxConfigBase::Get()->Read(wxT("/Processor/gui"),&t,HUGIN_PROCESSOR_GUI);
+        wxConfigBase::Get()->Read(wxT("/Processor/gui"), &t, HUGIN_PROCESSOR_GUI);
     };
     switch (t)
     {
         // PTBatcher
         case 0:
-            DoSendToBatch();
+            DoSendToBatch(userDefinedSetting);
             break;
         // hugin_stitch_project
         case 1:
-            DoStitch();
+            DoStitch(userDefinedSetting);
             break;
         // there is an error in the preferences
         default :
@@ -1485,6 +1422,11 @@ void PanoPanel::OnDoStitch ( wxCommandEvent & e )
       }
 }
 
+void PanoPanel::OnDoStitch(wxCommandEvent& e)
+{
+    DoStitchOrSendBatch();
+}
+
 void PanoPanel::FileFormatChanged(wxCommandEvent & e)
 {
 
@@ -1652,7 +1594,7 @@ bool PanoPanel::CheckGoodSize()
         return false;
     };
     wxString message;
-    unsigned long long int area = ((unsigned long int) cropped_region.width()) * ((unsigned long int) cropped_region.height());
+    const unsigned long long area = (static_cast<unsigned long long>(cropped_region.width()) * cropped_region.height());
     // Argh, more than half a gigapixel!
     if (area > 500000000)
     {
